Package: rms
Version: 8.1-1
Date: 2026-02-17
Title: Regression Modeling Strategies
Authors@R: person(given  = c("Frank E"),
                  family = "Harrell Jr",
                  role   = c("aut", "cre"),
                  email  = "fh@fharrell.com")
Depends: R (>= 4.4.0), Hmisc (>= 5.2-5)
Imports: methods, survival, quantreg, ggplot2, Matrix, SparseM, rpart,
        nlme (>= 3.1-123), polspline, multcomp, htmlTable (>= 1.11.0),
        htmltools, MASS, cluster, digest, colorspace, knitr, grDevices,
        scales
Suggests: boot, tcltk, plotly (>= 4.5.6), mice, icenReg, rmsb, nnet,
        VGAM, lattice, kableExtra
Description: Regression modeling, testing, estimation, validation,
	graphics, prediction, and typesetting by storing enhanced model design
	attributes in the fit.  'rms' is a collection of functions that
	assist with and streamline modeling.  It also contains functions for
	binary and ordinal logistic regression models, ordinal models for
  continuous Y with a variety of distribution families, and the Buckley-James
	multiple regression model for right-censored responses, and implements
	penalized maximum likelihood estimation for logistic and ordinary
	linear models.  'rms' works with almost any regression model, but it
	was especially written to work with binary or ordinal regression
	models, Cox regression, accelerated failure time models,
	ordinary linear models,	the Buckley-James model, generalized least
	squares for serially or spatially correlated observations, generalized
	linear models, and quantile regression.
License: GPL (>= 2)
URL: https://hbiostat.org/R/rms/, https://github.com/harrelfe/rms
LazyLoad: yes
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-02-17 12:22:16 UTC; harrelfe
Author: Frank E Harrell Jr [aut, cre]
Maintainer: Frank E Harrell Jr <fh@fharrell.com>
Repository: CRAN
Date/Publication: 2026-02-18 06:30:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 13:33:59 UTC; unix
Archs: rms.so.dSYM
