export(adapt_orm,annotateAnova,asis,bj,bjplot,bootBCa,bootcov,bootplot,bplot,calibrate,cph,catg,combineRelatedPredictors,confplot,contrast,coxphFit,cph,cr.setup,datadist,Design,DesignAssign,dxy.cens,effective.df,ExProb,fastbw,formatNP,gendata,ggplot,gIndex,Glm,Gls,groupkm,gTrans,Hazard,hazard.ratio.plot,histdensity,"%ia%",ie.setup,impactPO,infoMxop,intCalibration,interactions.containing,legend.nomabbrev,lm.pfit,lrm,lrm.fit,lrtest,LRupdate,lsp,matinv,matrx,modelData,Newlabels,Newlevels,nomogram,npsurv,Ocens,Ocens2ord,Ocens2Surv,Olinks,ols,ols.influence,oos.loglik,ordESS,orm,ordParallel,orm.fit,pantext,Penalty.matrix,Penalty.setup,pentrace,perimeter,perlcode,plotmathAnova,plot.contrast.rms,plot.lrm.partial,plotIntercepts,plot.xmean.ordinaly,pol,poma,pphsm,predab.resample,Predict,predictrms,prmiInfo,prModFit,probabilityFamilies,processMI,psm,rcs,recode2integer,related.predictors,reListclean,rexVar,robcov,Rq,sascode,scored,sensuc,setPb,show.influence,specs,strat,Surv,survreg.auxinfo,survdiffplot,survest,Survival,survplot,survplotp,univarLR,validate,val.prob,val.probg,val.surv,vif,which.influence,Xcontrast)

useDynLib(rms, .registration=TRUE, .fixes="F_")

import(Hmisc)
importFrom(survival, Surv, is.Surv, concordancefit, coxph.fit, coxph.control, survfit, survfit.formula, survfitKM, survfitcoxph.fit, survreg.fit, survreg.control, survpenal.fit, survreg.distributions, agreg.fit, agexact.fit, attrassign, untangle.specials)
importFrom(SparseM, solve, t, as.matrix)
#import(SparseM)

#import(ggplot2)
importFrom(ggplot2, ggplot, guides,  geom_point, geom_text, geom_col,
           xlab, ylab, labs, geom_tile, geom_segment, geom_step, scale_size, aes,
           facet_wrap, facet_grid, scale_color_manual, scale_shape_manual,
           ggplotGrob, coord_flip, element_blank, element_text, geom_path,
           geom_smooth, guide_legend, layer, layer_scales, scale_color_gradientn,
           scale_fill_brewer, scale_size_continuous, scale_size_manual,
           scale_x_continuous, scale_y_continuous, theme, unit, xlim, ylim,
           annotate, .data)

importFrom(cluster, ellipsoidhull)
importFrom(colorspace, rainbow_hcl)
importFrom(digest, digest)
importFrom(grDevices, n2mfrow)
importFrom(MASS, kde2d, bandwidth.nrd, cov.mve)
importFrom(scales, breaks_pretty, trans_new)

import(ggplot2)
#import(gridExtra)  # DOES NOT WORK UNLESS IN Depends IN DESCRIPTION
#importFrom(gridExtra, arrangeGrob)
importFrom(quantreg, rq.wfit, rq.fit, summary.rq)
import(nlme)
importFrom(rpart, rpart, rpart.control, prune)
importFrom(polspline, hare, phare)
import(multcomp)
# importFrom(multcomp, confint.glht, glht)  FAILS confint.glht not exported by namespace:multcomp
importFrom(htmlTable, htmlTable, txtRound)
importFrom(htmltools, HTML)
importFrom(knitr, kable)

importFrom(grDevices, dev.off, gray, grey, png, rgb, contourLines)
importFrom(graphics, abline, axis, boxplot, hist, legend,
           lines, locator, mtext, pairs, par, plot, plot.new,
           points, segments, strwidth, symbols, text,
           title, grconvertX, grconvertY, image)
importFrom(methods, existsFunction, getFunction, new)
importFrom(stats, .getXlevels, AIC, anova, approx, as.formula,
           asOneSidedFormula, binomial, coef, complete.cases, confint, contrasts, cor,
           dcauchy, delete.response, drop.terms, density, deviance,
           dnorm, family, fitted, formula, gaussian,
           glm, glm.control, glm.fit, influence.measures, is.empty.model, lm, lm.fit,
           lm.influence, lm.wfit, logLik, lowess, lsfit, makepredictcall,
           make.link, median, model.extract, model.frame,
           model.matrix, model.response,
           model.offset, model.weights, na.fail, na.omit, naresid,
           nlm, nlminb, nobs, optim, pcauchy, pchisq, pf, plogis,
           pnorm, predict, pt, qcauchy, qchisq, qlogis,
           qnorm, qqline, qqnorm, qt, quantile,
           reformulate, reshape, resid, residuals, residuals.glm, runif,
           sd, supsmu, terms, uniroot, update, var, vcov)
importFrom(utils, capture.output, de, getS3method, getFromNamespace, modifyList)


S3method(AIC, rms)
S3method(anova, rms)

S3method(calibrate, cph)
S3method(calibrate, default)
S3method(calibrate, orm)
S3method(calibrate, psm)

S3method(contrast, rms)

S3method(ExProb, orm)

S3method(Function, cph)
S3method(Function, rms)

S3method(Hazard, psm)

S3method(html, anova.rms)
S3method(html, naprint.delete)
S3method(html, summary.rms)
S3method(html, validate)


S3method(latex, anova.rms)
S3method(latex, bj)
S3method(latex, cph)
S3method(latex, Glm)
S3method(latex, Gls)
S3method(latex, lrm)
S3method(latex, naprint.delete)
S3method(latex, ols)
S3method(latex, orm)
S3method(latex, pphsm)
S3method(latex, psm)
S3method(latex, Rq)
S3method(latex, summary.rms)
S3method(latex, validate)

S3method(lines, residuals.psm.censored.normalized)

S3method(logLik, Gls)
S3method(logLik, ols)
S3method(logLik, rms)

S3method(makepredictcall, rms)

S3method(Mean,  cph)
S3method(Mean,  lrm)
S3method(Mean,  orm)
S3method(Mean,  psm)

S3method(Newlabels, rms)
S3method(Newlevels, rms)

S3method(nobs, rms)

S3method("[", Ocens)
S3method(as.data.frame, Ocens)
S3method(is.na, Ocens)

S3method(oos.loglik, ols)
S3method(oos.loglik, lrm)
S3method(oos.loglik, cph)
S3method(oos.loglik, psm)
S3method(oos.loglik, Glm)

S3method(plot, anova.rms)
S3method(plot, calibrate)
S3method(plot, calibrate.default)
S3method(plot, contrast.rms)
S3method(plot, ExProb)
S3method(plot, gIndex)
S3method(plot, lrm.partial)
S3method(plot, nomogram)
S3method(plot, pentrace)
S3method(plot, Predict)
S3method(plot, rexVar)
S3method(plot, sensuc)
S3method(plot, summary.rms)
S3method(plot, validate.rpart)
S3method(plot, val.prob)
S3method(plot, val.surv)
S3method(plot, val.survh)
S3method(plot, xmean.ordinaly)

S3method(plotp, Predict)

S3method(processMI, fit.mult.impute)

S3method(ggplot, npsurv)
S3method(ggplot, Predict)

S3method(predict, bj)
S3method(predict, cph)
S3method(predict, Glm)
S3method(predict, Gls)
S3method(predict, lrm)
S3method(predict, ols)
S3method(predict, orm)
S3method(predict, psm)
S3method(predict, Rq)

S3method(print, anova.rms)
S3method(print, bj)
S3method(print, calibrate)
S3method(print, contrast.rms)
S3method(print, cph)
S3method(print, datadist)
S3method(print, fastbw)
S3method(print, gIndex)
S3method(print, Glm)
S3method(print, Gls)
S3method(print, impactPO)
S3method(print, lrm)
S3method(print, lrtest)
S3method(print, nomogram)
S3method(print, Ocens)
S3method(print, ols)
S3method(print, orm)
S3method(print, pentrace)
S3method(print, pphsm)
S3method(print, Predict)
S3method(print, psm)
S3method(print, rexVar)
S3method(print, Rq)
S3method(print, specs.rms)
S3method(print, summary.rms)
S3method(print, summary.survreg2)
S3method(print, survest.psm)
S3method(print, validate)
S3method(print, validate.rpart)
S3method(print, val.prob)
S3method(print, val.survh)

S3method(Quantile, cph)
S3method(Quantile, orm)
S3method(Quantile, psm)

S3method(rbind, Predict)

S3method(residuals, bj)
S3method(residuals, cph)
S3method(residuals, Glm)
S3method(residuals, lrm)
S3method(residuals, ols)
S3method(residuals, orm)
S3method(residuals, psm)

S3method(specs, rms)
S3method(summary, rms)

S3method(survest, cph)
S3method(survest, orm)
S3method(survest, psm)

S3method(survfit, cph)

S3method(survplot, residuals.psm.censored.normalized)
S3method(survplot, rms)
S3method(survplot, npsurv)
S3method(survplot, orm)

S3method(survplotp, npsurv)

S3method(Survival, cph)
S3method(Survival, psm)
S3method(Survival, orm)

S3method(validate, bj)
S3method(validate, cph)
S3method(validate, lrm)
S3method(validate, ols)
S3method(validate, orm)
S3method(validate, psm)
S3method(validate, rpart)
S3method(validate, Rq)

S3method(vcov, cph)
S3method(vcov, Glm)
S3method(vcov, Gls)
S3method(vcov, lrm)
S3method(vcov, ols)
S3method(vcov, orm)
S3method(vcov, pphsm)
S3method(vcov, psm)
S3method(vcov, rms)

S3method("[", rms)
S3method("as.data.frame", rms)
