rms
=====

Regression Modeling Strategies

Current Goals
=============
* Implement estimation and prediction methods for the Bayesian partial
  proportional odds model `blrm` function

Web Sites
=============
* Overall: http://hbiostat.org/R/rms/
* Book: http://hbiostat.org/rms/
* CRAN: http://cran.r-project.org/web/packages/rms/
* Changelog: https://github.com/harrelfe/rms/commits/master/

To Do
=====
* Fix survplot so that explicitly named adjust-to values are still in subtitles.  See tests/cph2.s.
* Fix fit.mult.impute to average sigma^2 and then take square root, instead of averaging sigma
* Implement user-added distributions in psm - see https://github.com/harrelfe/rms/issues/41
