\name{rms-internal}
\title{Internal rms functions}
\alias{annotateAnova}
\alias{coxphFit}
\alias{lm.pfit}
\alias{ols.influence}
\alias{plotmathAnova}
\alias{probabilityFamilies}
\alias{prType}
\alias{as.data.frame.rms}
\alias{survreg.auxinfo}
\alias{val.probg}
\alias{quickRefit}
\description{Internal rms functions}
\details{These are not to be called by the user or are undocumented.}
\keyword{internal}
